@@ -67,6 +67,8 @@ struct _GtkSourceCompletionInfoPrivate
 	gulong focus_out_event_handler;
 
 	gint xoffset;
+
+	guint transient_set : 1;
 };
 
 /* Signals */
@@ -208,17 +210,21 @@ focus_out_event_cb (GtkSourceCompletionInfo *info)
 }
 
 static void
-update_attached_to (GtkSourceCompletionInfo *info)
+set_attached_to (GtkSourceCompletionInfo *info,
+		 GtkWidget               *attached_to)
 {
-	GtkWidget *attached_to = gtk_window_get_attached_to (GTK_WINDOW (info));
-
-	if (info->priv->focus_out_event_handler != 0 &&
-	    info->priv->attached_to != NULL)
+	if (info->priv->attached_to != NULL)
 	{
-		g_signal_handler_disconnect (info->priv->attached_to,
-					     info->priv->focus_out_event_handler);
+		g_object_remove_weak_pointer (G_OBJECT (info->priv->attached_to),
+					      (gpointer *) &info->priv->attached_to);
 
-		info->priv->focus_out_event_handler = 0;
+		if (info->priv->focus_out_event_handler != 0)
+		{
+			g_signal_handler_disconnect (info->priv->attached_to,
+						     info->priv->focus_out_event_handler);
+
+			info->priv->focus_out_event_handler = 0;
+		}
 	}
 
 	info->priv->attached_to = attached_to;
@@ -229,12 +235,21 @@ update_attached_to (GtkSourceCompletionInfo *info)
 	}
 
 	g_object_add_weak_pointer (G_OBJECT (attached_to),
-				   (gpointer) &info->priv->attached_to);
+				   (gpointer *) &info->priv->attached_to);
 
-	info->priv->focus_out_event_handler = g_signal_connect_swapped (attached_to,
-									"focus-out-event",
-									G_CALLBACK (focus_out_event_cb),
-									info);
+	info->priv->focus_out_event_handler =
+		g_signal_connect_swapped (attached_to,
+					  "focus-out-event",
+					  G_CALLBACK (focus_out_event_cb),
+					  info);
+
+	info->priv->transient_set = FALSE;
+}
+
+static void
+update_attached_to (GtkSourceCompletionInfo *info)
+{
+	set_attached_to (info, gtk_window_get_attached_to (GTK_WINDOW (info)));
 }
 
 static void
@@ -260,30 +275,41 @@ gtk_source_completion_info_init (GtkSourceCompletionInfo *info)
 }
 
 static void
-gtk_source_completion_info_finalize (GObject *object)
+gtk_source_completion_info_dispose (GObject *object)
 {
 	GtkSourceCompletionInfo *info = GTK_SOURCE_COMPLETION_INFO (object);
 
 	if (info->priv->idle_resize != 0)
 	{
 		g_source_remove (info->priv->idle_resize);
+		info->priv->idle_resize = 0;
 	}
 
-	if (info->priv->focus_out_event_handler != 0 &&
-	    info->priv->attached_to != NULL)
-	{
-		g_signal_handler_disconnect (info->priv->attached_to,
-					     info->priv->focus_out_event_handler);
-	}
+	set_attached_to (info, NULL);
 
-	G_OBJECT_CLASS (gtk_source_completion_info_parent_class)->finalize (object);
+	G_OBJECT_CLASS (gtk_source_completion_info_parent_class)->dispose (object);
 }
 
 static void
 gtk_source_completion_info_show (GtkWidget *widget)
 {
+	GtkSourceCompletionInfo *info = GTK_SOURCE_COMPLETION_INFO (widget);
+
 	/* First emit BEFORE_SHOW and then chain up */
-	g_signal_emit (widget, signals[BEFORE_SHOW], 0);
+	g_signal_emit (info, signals[BEFORE_SHOW], 0);
+
+	if (info->priv->attached_to != NULL && !info->priv->transient_set)
+	{
+		GtkWidget *toplevel;
+
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (info->priv->attached_to));
+		if (gtk_widget_is_toplevel (toplevel))
+		{
+			gtk_window_set_transient_for (GTK_WINDOW (info),
+						      GTK_WINDOW (toplevel));
+			info->priv->transient_set = TRUE;
+		}
+	}
 
 	GTK_WIDGET_CLASS (gtk_source_completion_info_parent_class)->show (widget);
 }
@@ -310,7 +336,7 @@ gtk_source_completion_info_class_init (GtkSourceCompletionInfoClass *klass)
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
-	object_class->finalize = gtk_source_completion_info_finalize;
+	object_class->dispose = gtk_source_completion_info_dispose;
 
 	widget_class->show = gtk_source_completion_info_show;
 	widget_class->draw = gtk_source_completion_info_draw;
@@ -403,6 +429,10 @@ compensate_for_gravity (GtkSourceCompletionInfo *window,
 		case GDK_GRAVITY_EAST:
 			*x = w;
 			break;
+		case GDK_GRAVITY_NORTH_WEST:
+		case GDK_GRAVITY_WEST:
+		case GDK_GRAVITY_SOUTH_WEST:
+		case GDK_GRAVITY_STATIC:
 		default:
 			*x = 0;
 			break;
@@ -421,6 +451,10 @@ compensate_for_gravity (GtkSourceCompletionInfo *window,
 		case GDK_GRAVITY_SOUTH_WEST:
 			*y = w;
 			break;
+		case GDK_GRAVITY_NORTH:
+		case GDK_GRAVITY_NORTH_EAST:
+		case GDK_GRAVITY_NORTH_WEST:
+		case GDK_GRAVITY_STATIC:
 		default:
 			*y = 0;
 			break;
@@ -532,6 +566,7 @@ gtk_source_completion_info_new (void)
 {
 	return g_object_new (GTK_SOURCE_TYPE_COMPLETION_INFO,
 	                     "type", GTK_WINDOW_POPUP,
+			     "border-width", 3,
 	                     NULL);
 }
 
